% TCO calculation in octave

addpath("c:/Octave/Octave-5.1.0.0/home/ocz/jsonlab-1.9.8")

% Input data

% Vehicle/charger input data
vehicle=loadjson('veh-ch-inp-data.json');
buscapcost12 = vehicle.buscapcost12;
buscapcost18 = vehicle.buscapcost18;
chargercost  = vehicle.chargercost;
chargerinst  = vehicle.chargerinst;
midlifecost  = vehicle.midlifecost;

% Simulation determined or known efficiencies
efficiency=loadjson('veh-eff.json');

efficiency12=efficiency.efficiency12;% kWh/km determined by microsimulation of characteristic full day duty cycles
efficiency18=efficiency.efficiency18;% kWh/km

% Energy cost
energy=loadjson('energy-cost.json');
avgfuelcost=energy.avgfuelcost;
avgmaincost=energy.avgmaincost;

% Fleet data
fleet =loadjson('fleet-data.json');
route =cell2mat(fleet.params.route{1,1});
peakheadway =cell2mat(fleet.params.peakheadway{1,1});
deptperhour =cell2mat(fleet.params.deptperhour{1,1});
rettriplen =cell2mat(fleet.params.rettriplen{1,1});
avgspeed =cell2mat(fleet.params.avgspeed{1,1});
articulated =cell2mat(fleet.params.articulated{1,1});


% Begin calculations
noofroutes  = size (route,2);
retcycletime = rettriplen*60 ./ avgspeed;
noofvecs     = ceil(retcycletime ./ peakheadway);

annualkmdriven=(rettriplen * 18 * 365) .* noofvecs;

busfleetreq18 = sum (noofvecs .* articulated);
busfleetreq12 = sum (noofvecs) - busfleetreq18;

energycons = annualkmdriven .* efficiency12;
energycons = energycons .+ articulated .* ( energycons * efficiency18/efficiency12 .- energycons);

annualfuelcosts = energycons * avgfuelcost;

%TCO per bus


annualkmdrivenperbus18 = sum (annualkmdriven .* articulated)/busfleetreq18;
annualkmdrivenperbus12 = (sum (annualkmdriven) - sum (annualkmdriven .* articulated))/busfleetreq12;

annualfuelcost18 =  annualkmdrivenperbus18 * avgfuelcost * efficiency18;
annualfuelcost12 =  annualkmdrivenperbus12 * avgfuelcost * efficiency12;

annualmaintcost18 = annualkmdrivenperbus18 * avgmaincost;
annualmaintcost12 = annualkmdrivenperbus12 * avgmaincost;


% TCO per 12 years and per bus
% assumption:
% number of routes = minimal number of chargers  (opp and depot)
% charger share rate = number of chargers / number of vehicles

chargersharerate = noofroutes / sum(noofvecs); 

Total12 = buscapcost12 + (chargercost + chargerinst)*chargersharerate  + midlifecost + annualfuelcost12 * 12 + annualmaintcost12 *12;
Total18 = buscapcost18 + (chargercost + chargerinst)*chargersharerate  + midlifecost + annualfuelcost18 * 12 + annualmaintcost18 *12;

AnnualTCO12  = Total12/12;
AnnualTCO18  = Total18/12;
MonthlyTCO12 = AnnualTCO12/12;
MonthlyTCO18 = AnnualTCO18/12;

AnnualEURperkm12= AnnualTCO12/annualkmdrivenperbus12;
AnnualEURperkm18= AnnualTCO18/annualkmdrivenperbus18;


% TCO per route

annualTCOperroute = annualkmdriven*AnnualEURperkm12;
annualTCOperroute = annualTCOperroute .+ articulated .* ( annualTCOperroute * AnnualEURperkm18/AnnualEURperkm12 .- annualTCOperroute);

outputmatrix1 = [route' peakheadway' deptperhour' rettriplen' avgspeed'];
outputmatrix2  = [route' annualTCOperroute' energycons' annualkmdriven'];


% TCO output sheet

format bank;
output_precision (0);
file_id = fopen('report.txt', 'w');

fdisp(file_id, "Total cost of ownership analysis");
fdisp(file_id, "Transit agency: Magdeburger Verkehrsbetriebe GmbH & Co. KG\n");

fdisp(file_id, "INPUT DATA\n");

fdisp(file_id, cstrcat("Capital cost of 12m bus     (EUR)  = ",num2str(buscapcost12)));
fdisp(file_id, cstrcat("Capital cost of 18m bus     (EUR)  = ",num2str(buscapcost18)));
fdisp(file_id, cstrcat("Capital cost charger equip  (EUR)  = ",num2str(chargercost)));
fdisp(file_id, cstrcat("Capital cost charger equip  (EUR)  = ",num2str(chargercost)));
fdisp(file_id, cstrcat("Install cost charger equip  (EUR)  = ",num2str(chargerinst)));
fdisp(file_id, cstrcat("Midlife cost                (EUR)  = ",num2str(midlifecost)));

fdisp(file_id, cstrcat("Efficiency of 12m bus    (kWh/km)  = ",num2str(efficiency12)));
fdisp(file_id, cstrcat("Efficiency of 18m bus    (kWh/km)  = ",num2str(efficiency18)));
fdisp(file_id, cstrcat("Average fuel cost        (EUR/kWh) = ",num2str(avgfuelcost)));
fdisp(file_id, cstrcat("Average maintenance cost (EUR/km)  = ",num2str(avgmaincost)));
fdisp(file_id, "\n");
fdisp(file_id, "Bus routes    Peak headway (mins)    Departures (per h)   Return trip length (km)   Avg speed (km/h)");
fdisp(file_id, outputmatrix1);
fdisp(file_id, "\n");

fdisp(file_id, "\nRESULTS for TOTAL COST of OWNERSHIP projection (approximated estimation)\n");
fdisp(file_id, "Bus routes    Yearly TCO (EUR)    Energy consumption (kWh)   km travelled");
fdisp(file_id, outputmatrix2);
fdisp(file_id, "\n");
format bank;
fdisp(file_id, cstrcat("Annual  TCO of 12m bus     (EUR)  = ",num2str(AnnualTCO12)));
fdisp(file_id, cstrcat("Monthly TCO of 12m bus     (EUR)  = ",num2str(MonthlyTCO12)));
fdisp(file_id, cstrcat("Annual  TCO of 18m bus     (EUR)  = ",num2str(AnnualTCO18)));
fdisp(file_id, cstrcat("Monthly TCO of 18m bus     (EUR)  = ",num2str(MonthlyTCO18)));
fdisp(file_id, cstrcat("Annual  EUR/km of 12m bus         = ",num2str(AnnualEURperkm12)));
fdisp(file_id, cstrcat("Annual  EUR/km of 18m bus         = ",num2str(AnnualEURperkm18)));

fdisp(file_id, "\nRESULTS for TOTAL COST of OWNERSHIP projection (approximated estimation)\n");





fclose(file_id);
